@@ -92,14 +92,9 @@ osmo_sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc)
 /*! \brief Bind a SCCP User to a given Point Code
  *  \param[in] inst SCCP Instance
  *  \param[in] name human-readable name
- *  \param[in] prim_cb User provided callback to pass a primitive/msg up the stack
  *  \param[in] ssn Sub-System Number to bind to
  *  \param[in] pc Point Code to bind to, or OSMO_SS7_PC_INVALID if none.
- *  \returns Callee-allocated SCCP User on success; negative otherwise
- *
- * Ownership of oph->msg in prim_cb is transferred to the user of the
- * registered callback when called.
- */
+ *  \returns Callee-allocated SCCP User on success; negative otherwise */
 static struct osmo_sccp_user *
 sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
 		  osmo_prim_cb prim_cb, uint16_t ssn, uint32_t pc)
@@ -132,14 +127,9 @@ sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
 /*! \brief Bind a given SCCP User to a given SSN+PC
  *  \param[in] inst SCCP Instance
  *  \param[in] name human-readable name
- *  \param[in] prim_cb User provided callback to pass a primitive/msg up the stack
  *  \param[in] ssn Sub-System Number to bind to
  *  \param[in] pc Point Code to bind to
- *  \returns Callee-allocated SCCP User on success; negative otherwise
- *
- * Ownership of oph->msg in prim_cb is transferred to the user of the
- * registered callback when called.
- */
+ *  \returns Callee-allocated SCCP User on success; negative otherwise */
 struct osmo_sccp_user *
 osmo_sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
 		       osmo_prim_cb prim_cb, uint16_t ssn, uint32_t pc)
@@ -150,13 +140,8 @@ osmo_sccp_user_bind_pc(struct osmo_sccp_instance *inst, const char *name,
 /*! \brief Bind a given SCCP User to a given SSN (at any PC)
  *  \param[in] inst SCCP Instance
  *  \param[in] name human-readable name
- *  \param[in] prim_cb User provided callback to pass a primitive/msg up the stack
  *  \param[in] ssn Sub-System Number to bind to
- *  \returns Callee-allocated SCCP User on success; negative otherwise
- *
- * Ownership of oph->msg in prim_cb is transferred to the user of the
- * registered callback when called.
- */
+ *  \returns Callee-allocated SCCP User on success; negative otherwise */
 struct osmo_sccp_user *
 osmo_sccp_user_bind(struct osmo_sccp_instance *inst, const char *name,
 		    osmo_prim_cb prim_cb, uint16_t ssn)
@@ -190,10 +175,7 @@ void *osmo_sccp_user_get_priv(struct osmo_sccp_user *scu)
 /*! \brief Send a SCCP User SAP Primitive up to the User
  *  \param[in] scu SCCP User to whom to send the primitive
  *  \param[in] prim Primitive to send to the user
- *  \returns return value of the SCCP User's prim_cb() function
- *
- * Ownership of prim->oph->msg is passed to the user of the registered callback
- */
+ *  \returns return value of the SCCP User's prim_cb() function */
 int sccp_user_prim_up(struct osmo_sccp_user *scu, struct osmo_scu_prim *prim)
 {
 	LOGP(DLSCCP, LOGL_DEBUG, "Delivering %s to SCCP User '%s'\n",
@@ -509,18 +491,6 @@ const char *osmo_sccp_user_name(struct osmo_sccp_user *scu)
  * Convenience function for CLIENT
  ***********************************************************************/
 
- /* Returns whether AS is already associated to any AS.
-  * Helper function for osmo_sccp_simple_client_on_ss7_id(). */
-static bool asp_serves_some_as(const struct osmo_ss7_asp *asp)
-{
-	struct osmo_ss7_as *as_i;
-	llist_for_each_entry(as_i, &asp->inst->as_list, list) {
-		if (osmo_ss7_as_has_asp(as_i, asp))
-			return true;
-	}
-	return false;
-}
-
 /*! \brief request an sccp client instance
  *  \param[in] ctx talloc context
  *  \param[in] ss7_id of the SS7/CS7 instance
@@ -640,23 +610,24 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 		/* Check if the user has created an ASP for this proto that is not added on any AS yet. */
 		struct osmo_ss7_asp *asp_i;
 		llist_for_each_entry(asp_i, &ss7->asp_list, list) {
+			struct osmo_ss7_as *as_i;
+			bool is_on_as = false;
 			if (asp_i->cfg.proto != prot)
 				continue;
-			if (asp_serves_some_as(asp_i)) {
-				/* This ASP is already on another AS.
-				 * If it was on this AS, we'd have found it above. */
+			llist_for_each_entry(as_i, &ss7->as_list, list) {
+				if (!osmo_ss7_as_has_asp(as_i, asp_i))
+					continue;
+				is_on_as = true;
+				break;
+			}
+			if (is_on_as) {
+				/* This ASP is already on another AS. If it was on this AS, we'd have found it above. */
 				continue;
 			}
 			/* This ASP matches the protocol and is not yet associated to any AS. Use it. */
 			asp = asp_i;
 			LOGP(DLSCCP, LOGL_NOTICE, "%s: ASP %s for %s is not associated with any AS, using it\n",
 			     name, asp->cfg.name, osmo_ss7_asp_protocol_name(prot));
-			ss7_as_add_asp(as, asp);
-			/* ASP became associated to a new AS, hence it needs to be
-			 * restarted to announce/register its Routing Context.
-			 * Make sure proper defaults are applied if app didn't
-			 * provide specific default values, then restart the ASP: */
-			ss7_asp_restart_after_reconfigure(asp);
 			break;
 		}
 		if (!asp) {
@@ -671,7 +642,6 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 			if (!asp)
 				goto out_rt;
 			asp_created = true;
-			asp->simple_client_allocated = true;
 			/* Ensure that the ASP we use is set to operate as a client. */
 			asp->cfg.is_server = false;
 			/* Ensure that the ASP we use is set to role ASP. */
@@ -680,11 +650,12 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 				ss7_asp_peer_set_hosts(&asp->cfg.local, asp, &default_local_ip, 1);
 			if (default_remote_ip)
 				ss7_asp_peer_set_hosts(&asp->cfg.remote, asp, &default_remote_ip, 1);
-			ss7_as_add_asp(as, asp);
-			/* Make sure proper defaults are applied if app didn't
-			provide specific default values, then restart the ASP: */
-			ss7_asp_restart_after_reconfigure(asp);
+			/* Make sure proper defaults are applied if app didn't provide specific default values */
+			ss7_asp_set_default_peer_hosts(asp);
+			asp->simple_client_allocated = true;
 		}
+
+		osmo_ss7_as_add_asp(as, asp->cfg.name);
 	}
 
 	/* Extra sanity checks if the ASP asp-clnt-* was pre-configured over VTY: */
@@ -725,9 +696,12 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 				goto out_asp;
 			}
 		}
-		/* ASP was already started here previously by VTY go_parent. */
 	}
 
+	/* Restart ASP */
+	if (prot != OSMO_SS7_ASP_PROT_IPA)
+		osmo_ss7_asp_use_default_lm(asp, LOGL_DEBUG);
+	osmo_ss7_asp_restart(asp);
 	LOGP(DLSCCP, LOGL_NOTICE, "%s: Using ASP instance %s\n", name,
 	     asp->cfg.name);
 
@@ -889,7 +863,7 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 		goto out_asp;
 	asp->cfg.is_server = true;
 	asp->cfg.role = OSMO_SS7_ASP_ROLE_SG;
-	ss7_as_add_asp(as, asp);
+	osmo_ss7_as_add_asp(as, asp_name);
 	talloc_free(asp_name);
 	talloc_free(as_name);
 	osmo_ss7_asp_restart(asp);
