@@ -27,7 +27,10 @@
 #include "xua_msg.h"
 #include <osmocom/sigtran/protocol/sua.h>
 
+#include "ss7_as.h"
+#include "ss7_asp.h"
 #include "ss7_internal.h"
+#include "ss7_xua_srv.h"
 #include "xua_asp_fsm.h"
 #include "xua_as_fsm.h"
 #include "xua_internal.h"
